from tuprolog import Info
from tuprolog.core import *
from tuprolog.core.comparators import *
from tuprolog.core.exception import *
from tuprolog.core.formatters import *
from tuprolog.core.operators import *
from tuprolog.core.visitors import *
from tuprolog.core.parsing import *
from tuprolog.unify import *
from tuprolog.theory import *
from tuprolog.theory.parsing import *
from tuprolog.solve import *
from tuprolog.solve.exception import *
from tuprolog.solve.exception.error import *
from tuprolog.solve.exception.warning import *
from tuprolog.solve.channel import *
from tuprolog.solve.flags import *
from tuprolog.solve.function import *
from tuprolog.solve.primitive import *
from tuprolog.solve.sideffcts import *
from tuprolog.solve.library import *
from tuprolog.solve.library.exception import *
from tuprolog.solve.data import *
from tuprolog.solve.stdlib import *
from tuprolog.solve.stdlib.primitive import *
from tuprolog.solve.stdlib.rule import *
from tuprolog.solve.stdlib.function import *
from tuprolog.solve.classic import *
from tuprolog.solve.prolog import *

import code

code.interact(local=locals())

input()